@@ -74,7 +74,7 @@ the displacements field, can be obtained as:
     \label{eq:fieldmap-4}\tag{4}
 
 This calculation is further complicated by the fact that :math:`\Theta_i`
-(and therfore, :math:`\Delta \Theta`) are clipped (or *wrapped*) within
+(and therefore, :math:`\Delta \Theta`) are clipped (or *wrapped*) within
 the range :math:`[0 \dotsb 2\pi )`.
 It is necessary to find the integer number of offsets that make a region
 continuously smooth with its neighbors (*phase-unwrapping*, [Jenkinson2003]_).
@@ -82,7 +82,7 @@ continuously smooth with its neighbors (*phase-unwrapping*, [Jenkinson2003]_).
 This corresponds to `this section of the BIDS specification
 <https://bids-specification.readthedocs.io/en/latest/04-modality-specific-files/01-magnetic-resonance-imaging-data.html#two-phase-images-and-two-magnitude-images>`__.
 Some scanners produce one ``phasediff`` map, where the drift between the two echos has
-already been calulated (see `the corresponding section of BIDS
+already been calculated (see `the corresponding section of BIDS
 <https://bids-specification.readthedocs.io/en/latest/04-modality-specific-files/01-magnetic-resonance-imaging-data.html#case-1-phase-difference-map-and-at-least-one-magnitude-image>`__).
 
 References
@@ -103,7 +103,7 @@ from niworkflows.engine.workflows import LiterateWorkflow as Workflow
 INPUT_FIELDS = ("magnitude", "fieldmap")
 
 
-def init_fmap_wf(omp_nthreads=1, debug=False, mode="phasediff", name="fmap_wf"):
+def init_fmap_wf(omp_nthreads=1, sloppy=False, debug=False, mode="phasediff", name="fmap_wf"):
     """
     Estimate the fieldmap based on a field-mapping MRI acquisition.
 
@@ -129,8 +129,10 @@ def init_fmap_wf(omp_nthreads=1, debug=False, mode="phasediff", name="fmap_wf"):
     ----------
     omp_nthreads : :obj:`int`
         Maximum number of threads an individual process may use.
+    sloppy : :obj:`bool`
+        Whether a fast but less accurate correction should be applied.
     debug : :obj:`bool`
-        Run on debug mode
+        Run in debug mode
     name : :obj:`str`
         Unique name of this workflow.
 
@@ -152,6 +154,8 @@ def init_fmap_wf(omp_nthreads=1, debug=False, mode="phasediff", name="fmap_wf"):
     fmap_mask : :obj:`str`
         Path to a binary brain mask corresponding to the ``fmap`` and ``fmap_ref``
         pair.
+    method: :obj:`str`
+        Short description of the estimation method that was run.
 
     """
     from ...interfaces.bspline import (
@@ -164,7 +168,7 @@ def init_fmap_wf(omp_nthreads=1, debug=False, mode="phasediff", name="fmap_wf"):
     workflow = Workflow(name=name)
     inputnode = pe.Node(niu.IdentityInterface(fields=INPUT_FIELDS), name="inputnode")
     outputnode = pe.Node(
-        niu.IdentityInterface(fields=["fmap", "fmap_ref", "fmap_mask", "fmap_coeff"]),
+        niu.IdentityInterface(fields=["fmap", "fmap_ref", "fmap_mask", "fmap_coeff", "method"]),
         name="outputnode",
     )
 
@@ -172,7 +176,7 @@ def init_fmap_wf(omp_nthreads=1, debug=False, mode="phasediff", name="fmap_wf"):
     bs_filter = pe.Node(BSplineApprox(), n_procs=omp_nthreads, name="bs_filter")
     bs_filter.interface._always_run = debug
     bs_filter.inputs.bs_spacing = (
-        [DEFAULT_LF_ZOOMS_MM, DEFAULT_HF_ZOOMS_MM] if not debug else [DEFAULT_ZOOMS_MM]
+        [DEFAULT_LF_ZOOMS_MM, DEFAULT_HF_ZOOMS_MM] if not sloppy else [DEFAULT_ZOOMS_MM]
     )
     bs_filter.inputs.extrapolate = not debug
 
@@ -197,6 +201,7 @@ phase-drift map(s) measure with two consecutive GRE (gradient-recalled echo)
 acquisitions.
 """
         phdiff_wf = init_phdiff_wf(omp_nthreads, debug=debug)
+        outputnode.inputs.method = "FMB (fieldmap-based) - phase-difference map"
 
         # fmt: off
         workflow.connect([
@@ -218,6 +223,7 @@ acquisitions.
 A *B<sub>0</sub>* nonuniformity map (or *fieldmap*) was directly measured with
 an MRI scheme designed with that purpose such as SEI (Spiral-Echo Imaging).
 """
+        outputnode.inputs.method = "FMB (fieldmap-based) - directly measured B0 map"
         # Merge input fieldmap images (assumes all are given in the same units!)
         fmapmrg = pe.Node(
             IntraModalMerge(zero_based_avg=False, hmc=False, to_ras=False),
@@ -336,7 +342,7 @@ def init_phdiff_wf(omp_nthreads, debug=False, name="phdiff_wf"):
     omp_nthreads : :obj:`int`
         Maximum number of threads an individual process may use
     debug : :obj:`bool`
-        Run on debug mode
+        Run in debug mode
     name : :obj:`str`
         Name of workflow (default: ``phdiff_wf``)
 
