@@ -23,7 +23,9 @@ A *B<sub>0</sub>*-nonuniformity map (or *fieldmap*) was estimated based on two (
 echo-planar imaging (EPI) references """
 
 
-def init_topup_wf(omp_nthreads=1, debug=False, name="pepolar_estimate_wf"):
+def init_topup_wf(
+    omp_nthreads=1, sloppy=False, debug=False, name="pepolar_estimate_wf"
+):
     """
     Create the PEPOLAR field estimation workflow based on FSL's ``topup``.
 
@@ -37,8 +39,10 @@ def init_topup_wf(omp_nthreads=1, debug=False, name="pepolar_estimate_wf"):
 
     Parameters
     ----------
-    debug : :obj:`bool`
+    sloppy : :obj:`bool`
         Whether a fast configuration of topup (less accurate) should be applied.
+    debug : :obj:`bool`
+        Run in debug mode
     name : :obj:`str`
         Name for this workflow
     omp_nthreads : :obj:`int`
@@ -62,12 +66,15 @@ def init_topup_wf(omp_nthreads=1, debug=False, name="pepolar_estimate_wf"):
         The path of mask corresponding to the ``fmap_ref`` output.
     fmap_coeff : :obj:`str` or :obj:`list` of :obj:`str`
         The path(s) of the B-Spline coefficients supporting the fieldmap.
+    method: :obj:`str`
+        Short description of the estimation method that was run.
 
     """
     from nipype.interfaces.fsl.epi import TOPUP
     from niworkflows.interfaces.nibabel import MergeSeries
     from niworkflows.interfaces.images import IntraModalMerge
 
+    from ...utils.misc import front as _front
     from ...interfaces.epi import GetReadoutTime
     from ...interfaces.utils import Flatten
     from ...interfaces.bspline import TOPUPCoeffReorient
@@ -89,10 +96,12 @@ def init_topup_wf(omp_nthreads=1, debug=False, name="pepolar_estimate_wf"):
                 "jacobians",
                 "xfms",
                 "out_warps",
+                "method",
             ]
         ),
         name="outputnode",
     )
+    outputnode.inputs.method = "PEB/PEPOLAR (phase-encoding based / PE-POLARity)"
 
     flatten = pe.Node(Flatten(), name="flatten")
     concat_blips = pe.Node(MergeSeries(), name="concat_blips")
@@ -105,7 +114,7 @@ def init_topup_wf(omp_nthreads=1, debug=False, name="pepolar_estimate_wf"):
 
     topup = pe.Node(
         TOPUP(
-            config=_pkg_fname("sdcflows", f"data/flirtsch/b02b0{'_quick' * debug}.cnf")
+            config=_pkg_fname("sdcflows", f"data/flirtsch/b02b0{'_quick' * sloppy}.cnf")
         ),
         name="topup",
     )
@@ -119,6 +128,11 @@ def init_topup_wf(omp_nthreads=1, debug=False, name="pepolar_estimate_wf"):
 
     brainextraction_wf = init_brainextraction_wf()
 
+    def _getfirstpe(in_meta):
+        if isinstance(in_meta, list):
+            in_meta = in_meta[0]
+        return in_meta["PhaseEncodingDirection"]
+
     # fmt: off
     workflow.connect([
         (inputnode, flatten, [("in_data", "in_data"),
@@ -129,13 +143,11 @@ def init_topup_wf(omp_nthreads=1, debug=False, name="pepolar_estimate_wf"):
         (flatten, topup, [(("out_meta", _pe2fsl), "encoding_direction")]),
         (readout_time, topup, [("readout_time", "readout_times")]),
         (concat_blips, topup, [("out_file", "in_file")]),
-        (topup, merge_corrected, [("out_corrected", "in_files")]),
-        (topup, fix_coeff, [("out_fieldcoef", "in_coeff"),
-                            ("out_corrected", "fmap_ref")]),
-        (topup, outputnode, [("out_field", "fmap"),
-                             ("out_jacs", "jacobians"),
-                             ("out_mats", "xfms"),
-                             ("out_warps", "out_warps")]),
+        (flatten, fix_coeff, [(("out_data", _front), "fmap_ref"),
+                              (("out_meta", _getfirstpe), "pe_dir")]),
+        (topup, fix_coeff, [("out_fieldcoef", "in_coeff")]),
+        (topup, outputnode, [("out_jacs", "jacobians"),
+                             ("out_mats", "xfms")]),
         (merge_corrected, brainextraction_wf, [("out_avg", "inputnode.in_file")]),
         (merge_corrected, outputnode, [("out_avg", "fmap_ref")]),
         (brainextraction_wf, outputnode, [("outputnode.out_mask", "fmap_mask")]),
@@ -143,6 +155,39 @@ def init_topup_wf(omp_nthreads=1, debug=False, name="pepolar_estimate_wf"):
     ])
     # fmt: on
 
+    if not debug:
+        # fmt: off
+        workflow.connect([
+            (topup, merge_corrected, [("out_corrected", "in_files")]),
+            (topup, outputnode, [("out_field", "fmap"),
+                                 ("out_warps", "out_warps")]),
+        ])
+        # fmt: on
+        return workflow
+
+    from ...interfaces.bspline import ApplyCoeffsField
+
+    unwarp = pe.Node(ApplyCoeffsField(), name="unwarp")
+    unwarp.interface._always_run = True
+
+    def _getpe(inlist):
+        if isinstance(inlist, dict):
+            inlist = [inlist]
+
+        return [m["PhaseEncodingDirection"] for m in inlist]
+
+    # fmt:off
+    workflow.connect([
+        (fix_coeff, unwarp, [("out_coeff", "in_coeff")]),
+        (flatten, unwarp, [("out_data", "in_target"),
+                           (("out_meta", _getpe), "pe_dir")]),
+        (readout_time, unwarp, [("readout_time", "ro_time")]),
+        (unwarp, outputnode, [("out_warp", "out_warps"),
+                              ("out_field", "fmap")]),
+        (unwarp, merge_corrected, [("out_corrected", "in_files")]),
+    ])
+    # fmt:on
+
     return workflow
 
 
